﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Windows.Forms;

using SFML.Window;
using SFML.Graphics;

namespace blendo_tgaview
{
    class Program
    {
        const int DEFAULTWIDTH = 800;
        const int DEFAULTHEIGHT = 600;

        RenderWindow app;
        RectangleShape shape;
        int mousescroll;

        static void Main(string[] args)
        {
            Program program = new Program();
            program.Go(args);
        }

        public void Go(string[] args)
        {
            if (args.Length <= 0)
            {
                
                MessageBox.Show("Click on .tga files to view them.\n\nNow exiting.", "Blendo TGA viewer");
                return;
                
                /*
                args = new string[1]
                {
                    "C:\\games\\qc1\\base\\textures\\glass\\glass1.tga"
                };*/
            }

            bool canScroll = true;
            int index;
            string[] files = Directory.GetFiles(Path.GetDirectoryName(args[0]), "*.tga");
            mousescroll = 0;

            //determine current index.
            index = GetIndex(args[0], files);


            Styles windowStyle = Styles.Close | Styles.Resize;

            app = new RenderWindow(new VideoMode(DEFAULTWIDTH, DEFAULTHEIGHT, 32), string.Empty, windowStyle);
            app.SetFramerateLimit(30);
            app.Closed += new EventHandler(app_Closed);
            app.Resized += new EventHandler<SizeEventArgs>(app_Resize);
            app.MouseWheelMoved += new EventHandler<MouseWheelEventArgs>(app_MouseWheelMoved);

            shape = LoadImage(args[0]);

            while (app.IsOpen())
            {
                app.DispatchEvents();
                app.Clear(new Color(64,64,64));

                if (Keyboard.IsKeyPressed(Keyboard.Key.Escape))
                {
                    app_Closed(null, null);
                }

                if (Mouse.IsButtonPressed(Mouse.Button.Right))
                {
                    MessageBox.Show("Blendo TGA viewer\n\nby Brendon Chung\nblendogames.com\n\nLeft/Right arrow = next picture\nLeft-click = next picture (click left side of screen to view previous image)\nESC = exit", "Blendo TGA viewer");
                }


                bool clickPrev = false;
                if (Mouse.IsButtonPressed(Mouse.Button.Left) && Mouse.GetPosition(app).X < app.Size.X / 2)
                {
                    clickPrev = true;
                }

                bool clickNext = false;
                if (Mouse.IsButtonPressed(Mouse.Button.Left) && Mouse.GetPosition(app).X > app.Size.X / 2)
                {
                    clickNext = true;
                }


                if ((Keyboard.IsKeyPressed(Keyboard.Key.Right) || clickNext || mousescroll < 0) && canScroll)
                {
                    canScroll = false;

                    index++;

                    if (index > files.Length - 1)
                    {
                        index = 0;
                    }

                    shape = LoadImage(files[index]);
                }
                else if ((Keyboard.IsKeyPressed(Keyboard.Key.Left) || clickPrev || mousescroll > 0) && canScroll)
                {
                    canScroll = false;

                    index--;

                    if (index < 0)
                    {
                        index = files.Length - 1;
                    }

                    shape = LoadImage(files[index]);
                }                

                if (!Keyboard.IsKeyPressed(Keyboard.Key.Left) && !Keyboard.IsKeyPressed(Keyboard.Key.Right) && !canScroll && !Mouse.IsButtonPressed(Mouse.Button.Left))
                {
                    canScroll = true;
                }

                mousescroll = 0;

                app.Draw(shape);

                app.Display();
            }
        }

        private int GetIndex(string path, string[] files)
        {
            for (int i = 0; i < files.Length; i++)
            {
                if (files[i] == path)
                {
                    return i;
                }
            }

            //Error! Just return first index.
            return 0;
        }

        private RectangleShape LoadImage(string path)
        {
            Image newimage;

            try
            {                
                newimage = new Image(path);
            }
            catch
            {
                MessageBox.Show(string.Format("Error loading image:\n{0}", path), "Blendo TGA viewer");
                return null;
            }

            RectangleShape newShape = new RectangleShape();
            newShape.Texture = new Texture(newimage);
            ResizeImage(newShape);

            app.SetTitle(string.Format("{0}             ({1}x{2})", Path.GetFileName(path), newShape.Texture.Size.X, newShape.Texture.Size.Y));

            return newShape;
        }

        private RectangleShape ResizeImage(RectangleShape shape)
        {
            float factor1 = app.Size.X / (float)shape.Texture.Size.X;

            if (shape.Texture.Size.Y * factor1 <= app.Size.Y)
            {
                shape.Size = new Vector2f(shape.Texture.Size.X * factor1, shape.Texture.Size.Y * factor1);

                //center it along the y axis.
                float delta1 = app.Size.Y - shape.Size.Y;
                shape.Position = new Vector2f(0, delta1 / 2);

                return shape;
            }

            float factor2 = app.Size.Y / (float)shape.Texture.Size.Y;
            shape.Size = new Vector2f(shape.Texture.Size.X * factor2, shape.Texture.Size.Y * factor2);

            //center it along the x axis.
            float delta2 = app.Size.X - shape.Size.X;
            shape.Position = new Vector2f(delta2 / 2, 0);

            return shape;
        }

        void app_MouseWheelMoved(object sender, MouseWheelEventArgs e)
        {
            mousescroll = e.Delta;
        }

        void app_Resize(object sender, SizeEventArgs e)
        {
            app.SetView(new SFML.Graphics.View(new Vector2f(e.Width/2, e.Height/2), new Vector2f(e.Width, e.Height)));
            shape = ResizeImage(shape);
        }

        void app_Closed(object sender, EventArgs e)
        {
            app.Close();
        }
    }
}